/******************************************************************
 *****                                                        *****
 *****  Name: easyweb.c                                       *****
 *****  Ver.: 1.0                                             *****
 *****  Date: 07/05/2001                                      *****
 *****  Auth: Andreas Dannenberg                              *****
 *****        HTWK Leipzig                                    *****
 *****        university of applied sciences                  *****
 *****        Germany                                         *****
 *****  Func: implements a dynamic HTTP-server by using       *****
 *****        the easyWEB-API                                 *****
 *****                                                        *****
 ******************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define extern            // Keil: Line added for modular project management

#include "easyweb.h"

#include "ENET.h"         // Keil: *.c -> *.h    // ethernet packet driver
#include "tcpip.h"        // Keil: *.c -> *.h    // easyWEB TCP/IP stack
#include <91x_lib.h>      // Keil: Register definition file for STR912

#include "webpage.h"                             // webside for our HTTP server (HTML)
// Piece of code number 1
#include "91x_mems.h"                            // driver for MEMs sensor
t_mems_data mems_data = {0,0,0,0,0,0};



int main(void)
{

  /* configure GPIO6 for LEDs */
  SCU->GPIOOUT[6]   = 0x0055;
  SCU->GPIOTYPE[6]   = 0x0F;
  GPIO6->DDR        = 0x0F;
  GPIO6->DR[0x0F*4] = 0x0F;	             // clear LEDs                     

  /* Push Button Setup */
  SCU->GPIOOUT[7]   = 0x0000;               /* P7.0..7 no output              */
  GPIO7->DDR        = 0x00;                 /* P7.0..7 Input (Push Buttons)   */

// Piece of code number 2
  Periph_Config_MEMs();                      // Configure peripherals for MEMs
  SPI_MEMS_Init();
 
  TCPLowLevelInit();

  GPIO6->DR[0x0F*4] = 0x0B;	             // Initialization finished

  HTTPStatus = 0;                                // clear HTTP-server's flag register

  TCPLocalPort = TCP_PORT_HTTP;                  // set port we want to listen to
  
  while (1)                                      // repeat forever
  {
    if (!(SocketStatus & SOCK_ACTIVE)) TCPPassiveOpen();   // listen for incoming TCP-connection
    DoNetworkStuff();                                      // handle network and easyWEB-stack
                                                           // events
    HTTPServer();
  }
}

// This function implements a very simple dynamic HTTP-server.
// It waits until connected, then sends a HTTP-header and the
// HTML-code stored in memory. Before sending, it replaces
// some special strings with dynamic values.
// NOTE: For strings crossing page boundaries, replacing will
// not work. In this case, simply add some extra lines
// (e.g. CR and LFs) to the HTML-code.

void HTTPServer(void)
{
  if (SocketStatus & SOCK_CONNECTED)             // check if somebody has connected to our TCP
  {
    if (SocketStatus & SOCK_DATA_AVAILABLE)      // check if remote TCP sent data
      TCPReleaseRxBuffer();                      // and throw it away

    if (SocketStatus & SOCK_TX_BUF_RELEASED)     // check if buffer is free for TX
    {
      if (!(HTTPStatus & HTTP_SEND_PAGE))        // init byte-counter and pointer to webside
      {                                          // if called the 1st time
        HTTPBytesToSend = sizeof(WebSide) - 1;   // get HTML length, ignore trailing zero
        PWebSide = (unsigned char *)WebSide;     // pointer to HTML-code
      }

      if (HTTPBytesToSend > MAX_TCP_TX_DATA_SIZE)     // transmit a segment of MAX_SIZE
      {
        if (!(HTTPStatus & HTTP_SEND_PAGE))           // 1st time, include HTTP-header
        {
          memcpy(TCP_TX_BUF, GetResponse, sizeof(GetResponse) - 1);
          memcpy(TCP_TX_BUF + sizeof(GetResponse) - 1, PWebSide, MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1);
          HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
          PWebSide += MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
        }
        else
        {
          memcpy(TCP_TX_BUF, PWebSide, MAX_TCP_TX_DATA_SIZE);
          HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE;
          PWebSide += MAX_TCP_TX_DATA_SIZE;
        }
          
        TCPTxDataCount = MAX_TCP_TX_DATA_SIZE;   // bytes to xfer
        InsertDynamicValues();                   // exchange some strings...
        TCPTransmitTxBuffer();                   // xfer buffer
      }
      else if (HTTPBytesToSend)                  // transmit leftover bytes
      {
        memcpy(TCP_TX_BUF, PWebSide, HTTPBytesToSend);
        TCPTxDataCount = HTTPBytesToSend;        // bytes to xfer
        InsertDynamicValues();                   // exchange some strings...
        TCPTransmitTxBuffer();                   // send last segment
        TCPClose();                              // and close connection
        HTTPBytesToSend = 0;                     // all data sent
      }

      HTTPStatus |= HTTP_SEND_PAGE;              // ok, 1st loop executed
    }
  }
  else
    HTTPStatus &= ~HTTP_SEND_PAGE;               // reset help-flag if not connected
}

// Piece of code number 3
// Converts MEMs data to special values for html output
unsigned char GetVal(unsigned char k1, unsigned char k2)
{
  unsigned  char c = 0;

  if (k1 == 'M') {
    switch (k2) {
      case 'X': {
        c = mems_data.outx_h;
        break;
      }
      case 'Y': {
        c = mems_data.outy_h;
        break;
      }
      case 'Z': {
        c = mems_data.outz_h;
        break;
      }
    }
  }

  if (k1 == 'M') {
    switch (k2) {
      case 'x': {
        c = mems_data.outx_l;
        break;
      }
      case 'y': {
        c = mems_data.outy_l;
        break;
      }
      case 'z': {
        c = mems_data.outz_l;
        break;
      }
    }
  }

  if (k1 == 'T') {
    switch (k2) {
      case 'X': {
        c = mems_data.outx_h;
        break;
      }
      case 'Y': {
        c = mems_data.outy_h;
        break;
      }
      case 'Z': {
        c = mems_data.outz_h;
        break;
      }
    }
    if (c > 100) c = 50+(256-c)*8;
      else c = 50-c*8;

  }



  return c;
 }


// searches the TX-buffer for special strings and replaces them
// with dynamic values

void InsertDynamicValues(void)
{
  unsigned char *Key;
           char NewKey[6];
  unsigned int i;

  if (TCPTxDataCount < 4) return;                     // there can't be any special string

  SPI_MEMS_Read(&mems_data);

  Key = TCP_TX_BUF;

  for (i = 0; i < (TCPTxDataCount - 3); i++)
  {
    if (*Key == '_')
      if (*(Key + 3) == '_') {
        sprintf(NewKey, "%4u", GetVal(*(Key+1),*(Key+2)));     // insert special value
        memcpy(Key, NewKey, 4);
    }
    Key++;
  }
}